// Title:	BiDiPro DLL module
// Desc:	W27E520
// Ver:		0.0.1
// Found:	2009-11-18
// Last:	2009-11-23
// CPU:		386+
// Model:	Tiny
// Compil:	Borland C++ 3.1 (DOS)
// Author:	Oleg O. Chukaev aka 0leg

#include <stdlib.h>
#include "../dlldata.h"
#include "../errors.h"
#include "../bidi.h"


#define dlp ((DLLDATA *) NULL)		/* ⥫    */
#define bdport dlp->bdprt		/* 㪠⥫  hardware ࠩ */


/* ७   CFG: */
static struct CFGKEY cfgkeys[] = {
	{'D', "tppw", 50, 50, 100000L},			//Write time (us)
	{'D', "tepw1", 50, 50, 100000L},		//Erase time 1 (us)
	{'D', "tepw2", 100000L, 100000L, 1000000L},	//Erase time 2 (us)
	{0, NULL, 0, 0, 0},				//End of table
};


#define cfg_tppw	cfgkeys[0].data
#define cfg_tepw1	cfgkeys[1].data
#define cfg_tepw2	cfgkeys[2].data

#define	PIN_ALE		0x10
#define	PIN_OE		0x20


static char *copyright="W27E520 EEPROM module (c) Oleg O. Chukaev  ["__DATE__"]";


/* 몫筨 ⠭ */
void power_off (void) {
	bdport (PRST);
}


/* 筨 ⠭ */
int power_on (char mode) {

	if (mode == 'R') {
		bdport (PFORCE | PWR1 | PBIN);		// DD1:OIO
		bdport (PFORCE | PWR2);			// DD4:OOO
		dlp->voltage (dlp->Vcc, 0, 0);
		bdport (PWB2 | 0x80);			// Vcc pin 36
		bdport (PWA2 | BITTER | PIN_OE);
	} else if (mode == 'W' || mode == 'E') {
		bdport (PFORCE | PWR1 | PBIN);		// DD1:OIO
		bdport (PFORCE | PWR2);			// DD4:OOO
		dlp->voltage (65, dlp->Vpp, dlp->Vid);
		bdport (PWB2 | 0x80);			// Vcc pin 36
		bdport (PWA2 | BITTER | PIN_OE);
	} else if (mode == 'I') {
		bdport (PFORCE | PWR1 | PBIN);		// DD1:OIO
		bdport (PFORCE | PWR2);			// DD4:OOO
		dlp->voltage (dlp->Vcc, dlp->Vpp, dlp->Vid);
		bdport (PWB2 | 0x80);			// Vcc pin 36
		bdport (PWA2 | BITTER | BITPE | PIN_OE);
		dlp->mcsdelay (100);
		bdport (PWB2 | 0x90);			// Vcc pin 36, Vid
	}
	dlp->mcsdelay (100000L);
	return 0;
}


/* ⥭, ࠢ, ஢ઠ  */
int read_all (void) {

	dword adr;
	dword istep = dlp->esize / 64L;
	dword errors = 0;
	byte rd;

	power_on ('R');
	for (adr = 0; adr < dlp->esize; adr++) {
		bdport (PFORCE | PWR1);			//OOO
		bdport (PWA2 | BITTER | PIN_OE | PIN_ALE);
		bdport (PWB1 | ((adr >> 0) & 0xff));	//A0 .. A7
		bdport (PWA2 | BITTER | PIN_OE);
		bdport (PFORCE | PWR1 | PBIN);		//OIO
		bdport (PWA2 | BITTER);
		bdport (PWC1 | ((adr >> 8) & 0xff));	//A8 .. A15
		rd = bdport (PRB1);
		bdport (PWA2 | BITTER | PIN_OE);

		switch (dlp->funcode) {
		case DFN_FREAD:
			dlp->setbytebuf (adr, rd);
			break;
		case DFN_FCOMPARE:
			if (dlp->getbytebuf (adr) != rd) {
				dlp->setbmark (adr, 1);
				errors++;
			}
			break;
		case DFN_FBLANK:
			if (rd != 0xff) {
				dlp->adr = adr;
				power_off ();
				return 1;
			}
		}

		if ((adr % istep == 0) && (dlp->iprogress (adr, 2))) {
			break;
		}
	}
	dlp->edata = errors;
	power_off ();
	return 0;
}


/* ࠭ */
int erase (void) {

	power_on ('E');
	bdport (PFORCE | PWR1);			//OOO
	bdport (PWA2 | BITTER | PIN_OE);

	bdport (PWA2 | BITTER | PIN_OE | PIN_ALE);
	bdport (PWB1 | 0x55);			//A0 .. A7
	bdport (PWA2 | BITTER | PIN_OE);
	bdport (PWC1 | 0x55);			//A15 .. A8
	bdport (PWB2 | 0x84);
	bdport (PWA2 | BITTER | BITPE | PIN_OE);
	bdport (PWB1 | 0xaa);			//A0 .. A7
	bdport (PWA2 | BITTER | BITPE | PIN_OE | PIN_ALE);
	dlp->mcsdelay (cfg_tepw1);
	bdport (PWA2 | BITTER | BITPE | PIN_OE);

	bdport (PWA2 | BITTER | PIN_OE);

	bdport (PWA2 | BITTER | PIN_OE | PIN_ALE);
	bdport (PWB1 | 0xaa);			//A0 .. A7
	bdport (PWA2 | BITTER | PIN_OE);
	bdport (PWC1 | 0x2a);			//A15 .. A8
	bdport (PWB2 | 0x84);
	bdport (PWA2 | BITTER | BITPE | PIN_OE);
	bdport (PWB1 | 0x10);			//A0 .. A7
	bdport (PWA2 | BITTER | BITPE | PIN_OE | PIN_ALE);
	dlp->mcsdelay (cfg_tepw2);
	bdport (PWA2 | BITTER | BITPE | PIN_OE);

	bdport (PWA2 | BITTER | PIN_OE);

	dlp->mcsdelay (100000L);

	return (0);
}


/*  */
int write_all (void) {

	dword adr;
	dword istep = dlp->esize / 64L;

	power_on ('W');
	bdport (PFORCE | PWR1);				//OOO
	for (adr = 0; adr < dlp->esize; adr++) {
		bdport (PWA2 | BITTER | PIN_OE | PIN_ALE);
		bdport (PWB1 | ((adr >> 0) & 0xff));	//A0 .. A7
		bdport (PWA2 | BITTER | PIN_OE);
		bdport (PWC1 | ((adr >> 8) & 0xff));	//A8 .. A15
		bdport (PWB2 | 0x84);
		bdport (PWA2 | BITTER | BITPE | PIN_OE);
		bdport (PWB1 | dlp->getbytebuf (adr));	//D0 .. D7
		bdport (PWA2 | BITTER | BITPE | PIN_OE | PIN_ALE);
		dlp->mcsdelay (cfg_tppw);
		bdport (PWA2 | BITTER | BITPE | PIN_OE);
		bdport (PWB2 | 0x80);
		bdport (PWA2 | BITTER | PIN_OE);
		if ((adr % istep == 0) && (dlp->iprogress (adr, 2))) {
			break;
		}
	}
	power_off ();
	return 0;
}


/* ⥭ ID */
int read_id (void) {

	dword id1, id2;

	power_on ('I');
	bdport (PFORCE | PWR1 | PBIN);			//OIO
	bdport (PWA2 | BITTER | BITPE | PIN_OE);

	bdport (PWC1 | 0);				//A8 = 0
	bdport (PWA2 | BITTER | BITPE);
	id1 = (byte)bdport (PRB1);			//Mfr
	bdport (PWA2 | BITTER | BITPE | PIN_OE);

	bdport (PWC1 | 1);				//A8 = 1
	bdport (PWA2 | BITTER | BITPE);
	id2 = (byte)bdport (PRB1);			//Dev
	bdport (PWA2 | BITTER | BITPE | PIN_OE);

	dlp->edata = id1 | (id2 << 16L);
	power_off ();
	return 0;
}


/* main code */
extern "C" int dmain (void) {

	switch (dlp->funcode) {
	case DFN_LOAD:
		// ⨢஢ 㯭 㭪樨:
		dlp->funmask[DFN_INIT] = 1;
		dlp->funmask[DFN_FREAD] = 1;
		dlp->funmask[DFN_FWRITE] = 1;
		dlp->funmask[DFN_FCOMPARE] = 1;
		dlp->funmask[DFN_READID] = 1;
		dlp->funmask[DFN_ERASE] = 1;
		dlp->funmask[DFN_FBLANK] = 1;
		dlp->funmask[DFN_TEST] = 0;
		dlp->defstate = 0xff;
		dlp->copyright = (byte *)copyright;
		dlp->version = DLLVERSION;
		dlp->cfgkeys = cfgkeys;
		return 0;

	case DFN_INIT:
		dlp->dippins = 20;
		return 0;

	case DFN_FREAD:
		return read_all ();

	case DFN_FWRITE:
		return write_all ();

	case DFN_FCOMPARE:
		return read_all ();

	case DFN_FBLANK:
		return read_all ();

	case DFN_ERASE:
		return erase ();

	case DFN_READID:
		return read_id ();

	default:
		return ERC_FUNUNSP;
	}
}

/* E0F */

